/*	Copyright (C) 2003-2010 Free Electron Organization
	Any use of this software requires a license.  If a valid license
	was not distributed with this file, visit freeelectron.org. */

#include "operator/operator.pmh"

namespace fe
{

void PartitionOp::initialize(void)
{
	catalog<String>("icon")="FE_alpha";

	catalog<String>("mode")="connectivity";
	catalog<String>("mode","label")="Mode";
	catalog<String>("mode","choice:0")="connectivity";
	catalog<String>("mode","label:0")="Connectivity";
	catalog<String>("mode","choice:1")="shader";
	catalog<String>("mode","label:1")="Shader";

	catalog<String>("class")="primitive";
	catalog<String>("class","label")="Class";
	catalog<String>("class","choice:0")="point";
	catalog<String>("class","label:0")="Point";
	catalog<String>("class","choice:1")="primitive";
	catalog<String>("class","label:1")="Primitive";

	catalog<String>("partitionAttr")="part";
	catalog<String>("partitionAttr","label")="Partition Attr";

	catalog<String>("partitionText")="PART";
	catalog<String>("partitionText","label")="Partition Text";
	catalog<bool>("partitionText","joined")=true;

	catalog<String>("pattern")="PART";
	catalog<String>("pattern","label")="Replace Pattern";
	catalog<String>("pattern","hint")=
			"Replace this string in the Partition Text with"
			" a string unique to each partition.";

	catalog< sp<Component> >("Input Surface");
}

void PartitionOp::handle(Record& a_rSignal)
{
	const String mode=catalog<String>("mode");
	const String partitionAttr=catalog<String>("partitionAttr");
	const String partitionText=catalog<String>("partitionText");
	const String pattern=catalog<String>("pattern");
	const String attrClass=catalog<String>("class");

	const Element element=(attrClass=="primitive")? e_primitive: e_point;

	sp<SurfaceAccessibleI> spOutputAccessible;
	if(!accessOutput(spOutputAccessible,a_rSignal)) return;

	sp<SurfaceAccessorI> spOutputPartition;
	if(!access(spOutputPartition,spOutputAccessible,
			element,partitionAttr)) return;

	sp<SurfaceAccessorI> spOutputVertices;
	if(!access(spOutputVertices,spOutputAccessible,
			e_primitive,e_vertices)) return;

	if(mode=="connectivity")
	{
		const SurfaceAccessibleI::AtomicChange atomicChange=
				SurfaceAccessibleI::e_pointsOfPrimitives;

		const String group;
		const U32 desiredCount=0;
		const BWORD checkPages=FALSE;

		sp<SurfaceRange> spSurfaceRange=
				spOutputAccessible->atomizeConnectivity(
				atomicChange,group,desiredCount,checkPages);

//		feLog("PartitionOp::handle count %d SurfaceRange\n%s\n",
//				spOutputPartition->count(),
//				spSurfaceRange->dump().c_str());

		const U32 atomicCount=spSurfaceRange->atomicCount();
		for(U32 atomicIndex=0;atomicIndex<atomicCount;atomicIndex++)
		{
			String number;
			number.sPrintf("%d",atomicIndex);
			const String partition=partitionText.replace(pattern,number);

			const SurfaceRange::MultiSpan& rMultiSpan=
					spSurfaceRange->atomic(atomicIndex);

			const U32 spanCount=rMultiSpan.spanCount();
			for(U32 spanIndex=0;spanIndex<spanCount;spanIndex++)
			{
				const SurfaceRange::Span& rSpan=rMultiSpan.span(spanIndex);
				const U32 start=rSpan[0];
				const U32 end=rSpan[1];

				for(U32 primitiveIndex=start;primitiveIndex<=end;
						primitiveIndex++)
				{
					if(attrClass=="primitive")
					{
						spOutputPartition->set(primitiveIndex,partition);
					}
					else
					{
						const U32 subCount=
								spOutputVertices->subCount(primitiveIndex);
						for(U32 subIndex=0;subIndex<subCount;subIndex++)
						{
							const U32 pointIndex=spOutputVertices->integer(
									primitiveIndex,subIndex);
							spOutputPartition->set(pointIndex,partition);
						}
					}
				}
			}
		}
		return;
	}

	if(mode=="shader")
	{
		sp<SurfaceAccessorI> spOutputShader;
		if(!access(spOutputShader,spOutputAccessible,
				e_primitive,"<shader>")) return;

		const U32 primitiveCount=spOutputShader->count();
		for(U32 primitiveIndex=0;primitiveIndex<primitiveCount;primitiveIndex++)
		{
			const String shaderName=spOutputShader->string(primitiveIndex);
			const String partition=partitionText.replace(pattern,shaderName);

//			feLog("PartitionOp::handle primitive %d/%d shader \"%s\"\n",
//					primitiveIndex,primitiveCount,shaderName.c_str());

			if(attrClass=="primitive")
			{
				spOutputPartition->set(primitiveIndex,partition);
			}
			else
			{
				const U32 subCount=spOutputVertices->subCount(primitiveIndex);
				for(U32 subIndex=0;subIndex<subCount;subIndex++)
				{
					const U32 pointIndex=
							spOutputVertices->integer(primitiveIndex,subIndex);
					spOutputPartition->set(pointIndex,partition);
				}
			}
		}

		return;
	}
}

} // namespace
