/*	Copyright (C) 2003-2010 Free Electron Organization
	Any use of this software requires a license.  If a valid license
	was not distributed with this file, visit freeelectron.org. */

/** @file */

#define FE_SHO_DEBUG		FALSE

#include "operator/operator.pmh"

using namespace fe;

void SpreadsheetOp::initialize(void)
{
	catalog<String>("icon")="FE_alpha";

	catalog<String>("attributes")="";
	catalog<String>("attributes","label")="Attributes";

	catalog<String>("class")="point";
	catalog<String>("class","label")="Class";
	catalog<String>("class","choice:0")="point";
	catalog<String>("class","label:0")="Point";
	catalog<String>("class","choice:1")="primitive";
	catalog<String>("class","label:1")="Primitive";

	catalog<I32>("start")=0;
	catalog<I32>("start","high")=100;

	catalog<I32>("length")=10;
	catalog<I32>("length","high")=100;

	catalog< sp<Component> >("Brush");
	catalog<String>("Brush","implementation")="DrawI";

	catalog< sp<Component> >("Input Surface");

	m_spSolid=new DrawMode();
	m_spSolid->setDrawStyle(DrawMode::e_solid);
}

void SpreadsheetOp::handle(Record& a_rSignal)
{
	String attributes=catalog<String>("attributes");
	const String attrClass=catalog<String>("class");
	const I32 start=catalog<I32>("start");
	const I32 length=catalog<I32>("length");

	catalog< sp<Component> >("Brush");
	sp<DrawI> spDrawOverlay;
	accessBrushOverlay(spDrawOverlay,a_rSignal,e_quiet);

	if(spDrawOverlay.isValid())
	{
#if FE_SHO_DEBUG
		feLog("SpreadsheetOp::handle brush\n");
#endif
		m_brushed=TRUE;

		spDrawOverlay->pushDrawMode(m_spSolid);

		m_event.bind(windowEvent(a_rSignal));
		const I32 mouseX=m_event.mouseX();
		const I32 mouseY=m_event.mouseY();

		const Color red(1.0,0.0,0.0,1.0);


		spDrawOverlay->popDrawMode();

		return;
	}

	const BWORD paramChanged=!m_brushed;
	m_brushed=FALSE;

#if FE_SHO_DEBUG
	feLog("SpreadsheetOp::handle changed %d\n",paramChanged);
#endif

	if(!paramChanged)
	{
		return;
	}

	const Element element=attrClass=="point"? e_point: e_primitive;

	sp<SurfaceAccessibleI> spInputSurface;
	access(spInputSurface,"Input Surface");

	sp<SurfaceAccessorI> spInputElement;
	access(spInputElement,spInputSurface,element,e_generic,e_warning);

	const U32 elementCount=spInputElement->count();

	U32 elementMax=start+length;
	if(elementMax>elementCount)
	{
		elementMax=elementCount;
	}

	feLog("\n%16s","");
	for(U32 elementIndex=start;elementIndex<elementMax;elementIndex++)
	{
		feLog("%10d",elementIndex);
	}
	feLog("\n");

	String attrName;
	while(!(attrName=attributes.parse()).empty())
	{
		access(spInputElement,spInputSurface,element,attrName,e_warning);

		const String attrType=spInputElement->type();
		if(attrType=="string")
		{
			feLog("%16s",attrName.c_str());

			for(U32 elementIndex=start;elementIndex<elementMax;elementIndex++)
			{
				const String value=spInputElement->string(elementIndex);
				feLog("%10s",value.c_str());
			}

			feLog("\n");
		}
		else if(attrType=="integer")
		{
			feLog("%16s",attrName.c_str());

			for(U32 elementIndex=start;elementIndex<elementMax;elementIndex++)
			{
				const I32 value=spInputElement->integer(elementIndex);
				feLog("%10d",value);
			}

			feLog("\n");
		}
		else if(attrType=="real")
		{
			feLog("%16s",attrName.c_str());

			for(U32 elementIndex=start;elementIndex<elementMax;elementIndex++)
			{
				const Real value=spInputElement->real(elementIndex);
				feLog("%10.4G",value);
			}

			feLog("\n");
		}
		else if(attrType=="vector3")
		{
			for(U32 m=0;m<3;m++)
			{
				feLog("%13s[%d]",attrName.c_str(),m);
				for(U32 elementIndex=start;elementIndex<elementMax;
						elementIndex++)
				{
					const SpatialVector value=
							spInputElement->spatialVector(elementIndex);
					feLog("%10.4G",value[m]);
				}

				feLog("\n");
			}
		}
	}
}
